﻿
using System;
using Microsoft.Xna.Framework;

namespace Silo {

	public class Framerate {
		int frame;
		DateTime start;
		TimeSpan updateInterval = TimeSpan.FromSeconds(1);
		TimeSpan recentInterval = TimeSpan.FromSeconds(1);
		int recentFrames;
		public event EventHandler Updated;

		public Framerate() {
			Clear();
		}
		public void Clear() {
			frame = 0;
			start = DateTime.Now;
		}
		public bool Update() {
			DateTime current = DateTime.Now;
			TimeSpan span = current - start;
			if (span >= updateInterval) {
				recentInterval = span;
				recentFrames = frame;
				start = current;
				frame = 0;
				if (Updated != null)
					Updated(this, EventArgs.Empty);
				return true;
			}
			++frame;
			return false;
		}
		public TimeSpan UpdateInterval { get { return updateInterval; } }
		public TimeSpan RecentInterval { get { return recentInterval; } }
		public int RecentFrames { get { return recentFrames; } }
	}

	public class FramerateDisplay : TextComponent {
		readonly Framerate updateFramerate = new Framerate();
		readonly Framerate drawFramerate = new Framerate();

		public FramerateDisplay(Game game) : base(game) { }
		protected override void LoadContent() {
			updateFramerate.Clear();
			drawFramerate.Clear();
			base.LoadContent();
		}
		public override void Update(GameTime gameTime) {
			if (updateFramerate.Update() |drawFramerate.Update()) {
				Text = string.Format("Update: {0:###0.00} fps,\nDraw: {0:###0.00} fps",
					updateFramerate.RecentFrames / updateFramerate.RecentInterval.TotalSeconds);
			}
			base.Update(gameTime);
		}
		public Framerate UpdateFramerate { get { return updateFramerate; } }
		public Framerate DrawFramerate { get { return drawFramerate; } }
	}

}
