﻿
using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Graphics.PackedVector;

namespace Silo {

	public class SiloGame : Game {
		GraphicsDeviceManager graphics;
		SiloField field;
		TextComponent text;
		FramerateDisplay framerate;
		InputState input = new InputState();

		public SiloGame() {
			graphics = new GraphicsDeviceManager(this);
			Content.RootDirectory = "Content";
			int width = 640;
			int height = 480;
			this.graphics.PreferredBackBufferWidth = width;
			this.graphics.PreferredBackBufferHeight = height;
			Components.Add(field = new SiloField(this, width, height));
			Components.Add(text = new TextComponent(this));
			Components.Add(framerate = new FramerateDisplay(this));
			framerate.DrawOrder = 0x40000000;
			framerate.Alignment = Alignment.Bottom;
			framerate.Position = new Vector2(64, -64);
			framerate.Color = new Color(new Vector4(0, 0.9f, 0.4f, 0.7f));
			framerate.UpdateFramerate.Updated += UpdateInfo;
			text.DrawOrder = 0x10;
			text.Position = new Vector2(64, 64);
			text.Color = new Color(new Vector4(1, 1, 1, 0.7f));

			graphics.SynchronizeWithVerticalRetrace = false;
			IsFixedTimeStep = false;
			IsMouseVisible = true;

			field.Resized += UpdateInfo;
			UpdateInfo(this, EventArgs.Empty);
		}

		void UpdateInfo(object sender, EventArgs e) {
			text.Text = string.Format("silo {0}x{1}\n{2:#,0} pps", field.Width, field.Height,
				(field.Width * field.Height * framerate.UpdateFramerate.RecentFrames) / framerate.UpdateFramerate.RecentInterval.TotalSeconds);
		}
		protected override void Initialize() {
			base.Initialize();
		}
		protected override void LoadContent() {
			framerate.Font = text.Font = Content.Load<SpriteFont>("font/Default");
			base.LoadContent();
		}
		protected override void Update(GameTime gameTime) {
			input.Update();
			if ((input.Down & InputButtons.Left) != 0)
				field.Initialize(Math.Max(1, field.Width >> 1), Math.Max(1, field.Height >> 1));
			if ((input.Down & InputButtons.Right) != 0) {
				Viewport viewport = graphics.GraphicsDevice.Viewport;
				field.Initialize(Math.Min(viewport.Width, field.Width << 1), Math.Min(viewport.Height, field.Height << 1));
			}
			if ((input.Down & InputButtons.Reset) != 0)
				field.Reset();
			if ((input.Down & InputButtons.Exit) != 0)
				Exit();
			if ((input.Down & InputButtons.ToggleScreen) != 0)
				graphics.ToggleFullScreen();

			KeyboardState keyboardState = Keyboard.GetState();
#if XBOX
#else
			MouseState mouseState = Mouse.GetState();
			if (mouseState.LeftButton == ButtonState.Pressed) {
				if (keyboardState.IsKeyDown(Keys.LeftControl)) {
					Viewport viewport = graphics.GraphicsDevice.Viewport;
					field.Pen.Pressed = true;
					field.Pen.Position = new Vector2(mouseState.X / (float)viewport.Width, mouseState.Y / (float)viewport.Height);
					field.Pen.Color = new Vector4(+1, -1, 0, 0);
					field.Pen.Radius = 4.0f / field.Height;
				} else {
					Viewport viewport = graphics.GraphicsDevice.Viewport;
					field.Pen.Pressed = true;
					field.Pen.Position = new Vector2(mouseState.X / (float)viewport.Width, mouseState.Y / (float)viewport.Height);
					field.Pen.Color = new Vector4(-1, +1, 0, 0);
					field.Pen.Radius = 4.0f / field.Height;
				}
			} else if (mouseState.RightButton == ButtonState.Pressed) {
				Viewport viewport = graphics.GraphicsDevice.Viewport;
				field.Pen.Pressed = true;
				field.Pen.Position = new Vector2(mouseState.X / (float)viewport.Width, mouseState.Y / (float)viewport.Height);
				field.Pen.Color = new Vector4(-1, -1, 0, 0);
				field.Pen.Radius = 4.0f / field.Height;
			} else {
				field.Pen.Pressed = false;
			}
#endif
			base.Update(gameTime);
		}
		protected override void Draw(GameTime gameTime) {
			graphics.GraphicsDevice.Clear(ClearOptions.DepthBuffer, Color.Black, 1.0f, 0);
			base.Draw(gameTime);
		}
	}
}
